/*
 * Generated by asn1c-0.9.22 (http://lionet.info/asn1c)
 * From ASN.1 module "RRLP-Components"
 * 	found in "../rrlp-components.asn"
 */

#ifndef	_GPSTime_H_
#define	_GPSTime_H_


#include <asn_application.h>

/* Including external dependencies */
#include "GPSTOW23b.h"
#include "GPSWeek.h"
#include <constr_SEQUENCE.h>

#ifdef __cplusplus
extern "C" {
#endif

/* GPSTime */
typedef struct GPSTime {
	GPSTOW23b_t	 gpsTOW23b;
	GPSWeek_t	 gpsWeek;
	
	/* Context for parsing across buffer boundaries */
	asn_struct_ctx_t _asn_ctx;
} GPSTime_t;

/* Implementation */
extern asn_TYPE_descriptor_t asn_DEF_GPSTime;

#ifdef __cplusplus
}
#endif

#endif	/* _GPSTime_H_ */
#include <asn_internal.h>
