/*
 * Generated by asn1c-0.9.22 (http://lionet.info/asn1c)
 * From ASN.1 module "SUPL-POS"
 * 	found in "../supl-pos.asn"
 */

#ifndef	_PosPayLoad_H_
#define	_PosPayLoad_H_


#include <asn_application.h>

/* Including external dependencies */
#include <OCTET_STRING.h>
#include <constr_CHOICE.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Dependencies */
typedef enum PosPayLoad_PR {
	PosPayLoad_PR_NOTHING,	/* No components present */
	PosPayLoad_PR_tia801payload,
	PosPayLoad_PR_rrcPayload,
	PosPayLoad_PR_rrlpPayload,
	/* Extensions may appear below */
	
} PosPayLoad_PR;

/* PosPayLoad */
typedef struct PosPayLoad {
	PosPayLoad_PR present;
	union PosPayLoad_u {
		OCTET_STRING_t	 tia801payload;
		OCTET_STRING_t	 rrcPayload;
		OCTET_STRING_t	 rrlpPayload;
		/*
		 * This type is extensible,
		 * possible extensions are below.
		 */
	} choice;
	
	/* Context for parsing across buffer boundaries */
	asn_struct_ctx_t _asn_ctx;
} PosPayLoad_t;

/* Implementation */
extern asn_TYPE_descriptor_t asn_DEF_PosPayLoad;

#ifdef __cplusplus
}
#endif

#endif	/* _PosPayLoad_H_ */
#include <asn_internal.h>
