/*
 * Generated by asn1c-0.9.22 (http://lionet.info/asn1c)
 * From ASN.1 module "RRLP-Components"
 * 	found in "../rrlp-components.asn"
 */

#ifndef	_NeighborIdentity_H_
#define	_NeighborIdentity_H_


#include <asn_application.h>

/* Including external dependencies */
#include "BSICAndCarrier.h"
#include "CellID.h"
#include "MultiFrameCarrier.h"
#include "RequestIndex.h"
#include "SystemInfoIndex.h"
#include "CellIDAndLAC.h"
#include <constr_CHOICE.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Dependencies */
typedef enum NeighborIdentity_PR {
	NeighborIdentity_PR_NOTHING,	/* No components present */
	NeighborIdentity_PR_bsicAndCarrier,
	NeighborIdentity_PR_ci,
	NeighborIdentity_PR_multiFrameCarrier,
	NeighborIdentity_PR_requestIndex,
	NeighborIdentity_PR_systemInfoIndex,
	NeighborIdentity_PR_ciAndLAC
} NeighborIdentity_PR;

/* NeighborIdentity */
typedef struct NeighborIdentity {
	NeighborIdentity_PR present;
	union NeighborIdentity_u {
		BSICAndCarrier_t	 bsicAndCarrier;
		CellID_t	 ci;
		MultiFrameCarrier_t	 multiFrameCarrier;
		RequestIndex_t	 requestIndex;
		SystemInfoIndex_t	 systemInfoIndex;
		CellIDAndLAC_t	 ciAndLAC;
	} choice;
	
	/* Context for parsing across buffer boundaries */
	asn_struct_ctx_t _asn_ctx;
} NeighborIdentity_t;

/* Implementation */
extern asn_TYPE_descriptor_t asn_DEF_NeighborIdentity;

#ifdef __cplusplus
}
#endif

#endif	/* _NeighborIdentity_H_ */
#include <asn_internal.h>
