/*
 * Generated by asn1c-0.9.22 (http://lionet.info/asn1c)
 * From ASN.1 module "RRLP-Components"
 * 	found in "../rrlp-components.asn"
 */

#include "GANSSTOD-GSMTimeAssociation.h"

static asn_TYPE_member_t asn_MBR_GANSSTOD_GSMTimeAssociation_1[] = {
	{ ATF_NOFLAGS, 0, offsetof(struct GANSSTOD_GSMTimeAssociation, bcchCarrier),
		(ASN_TAG_CLASS_CONTEXT | (0 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_BCCHCarrier,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"bcchCarrier"
		},
	{ ATF_NOFLAGS, 0, offsetof(struct GANSSTOD_GSMTimeAssociation, bsic),
		(ASN_TAG_CLASS_CONTEXT | (1 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_BSIC,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"bsic"
		},
	{ ATF_NOFLAGS, 0, offsetof(struct GANSSTOD_GSMTimeAssociation, frameNumber),
		(ASN_TAG_CLASS_CONTEXT | (2 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_FrameNumber,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"frameNumber"
		},
	{ ATF_NOFLAGS, 0, offsetof(struct GANSSTOD_GSMTimeAssociation, timeSlot),
		(ASN_TAG_CLASS_CONTEXT | (3 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_TimeSlot,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"timeSlot"
		},
	{ ATF_NOFLAGS, 0, offsetof(struct GANSSTOD_GSMTimeAssociation, bitNumber),
		(ASN_TAG_CLASS_CONTEXT | (4 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_BitNumber,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"bitNumber"
		},
	{ ATF_POINTER, 1, offsetof(struct GANSSTOD_GSMTimeAssociation, frameDrift),
		(ASN_TAG_CLASS_CONTEXT | (5 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_FrameDrift,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"frameDrift"
		},
};
static int asn_MAP_GANSSTOD_GSMTimeAssociation_oms_1[] = { 5 };
static ber_tlv_tag_t asn_DEF_GANSSTOD_GSMTimeAssociation_tags_1[] = {
	(ASN_TAG_CLASS_UNIVERSAL | (16 << 2))
};
static asn_TYPE_tag2member_t asn_MAP_GANSSTOD_GSMTimeAssociation_tag2el_1[] = {
    { (ASN_TAG_CLASS_CONTEXT | (0 << 2)), 0, 0, 0 }, /* bcchCarrier at 1118 */
    { (ASN_TAG_CLASS_CONTEXT | (1 << 2)), 1, 0, 0 }, /* bsic at 1119 */
    { (ASN_TAG_CLASS_CONTEXT | (2 << 2)), 2, 0, 0 }, /* frameNumber at 1120 */
    { (ASN_TAG_CLASS_CONTEXT | (3 << 2)), 3, 0, 0 }, /* timeSlot at 1121 */
    { (ASN_TAG_CLASS_CONTEXT | (4 << 2)), 4, 0, 0 }, /* bitNumber at 1122 */
    { (ASN_TAG_CLASS_CONTEXT | (5 << 2)), 5, 0, 0 } /* frameDrift at 1123 */
};
static asn_SEQUENCE_specifics_t asn_SPC_GANSSTOD_GSMTimeAssociation_specs_1 = {
	sizeof(struct GANSSTOD_GSMTimeAssociation),
	offsetof(struct GANSSTOD_GSMTimeAssociation, _asn_ctx),
	asn_MAP_GANSSTOD_GSMTimeAssociation_tag2el_1,
	6,	/* Count of tags in the map */
	asn_MAP_GANSSTOD_GSMTimeAssociation_oms_1,	/* Optional members */
	1, 0,	/* Root/Additions */
	-1,	/* Start extensions */
	-1	/* Stop extensions */
};
asn_TYPE_descriptor_t asn_DEF_GANSSTOD_GSMTimeAssociation = {
	"GANSSTOD-GSMTimeAssociation",
	"GANSSTOD-GSMTimeAssociation",
	SEQUENCE_free,
	SEQUENCE_print,
	SEQUENCE_constraint,
	SEQUENCE_decode_ber,
	SEQUENCE_encode_der,
	SEQUENCE_decode_xer,
	SEQUENCE_encode_xer,
	SEQUENCE_decode_uper,
	SEQUENCE_encode_uper,
	0,	/* Use generic outmost tag fetcher */
	asn_DEF_GANSSTOD_GSMTimeAssociation_tags_1,
	sizeof(asn_DEF_GANSSTOD_GSMTimeAssociation_tags_1)
		/sizeof(asn_DEF_GANSSTOD_GSMTimeAssociation_tags_1[0]), /* 1 */
	asn_DEF_GANSSTOD_GSMTimeAssociation_tags_1,	/* Same as above */
	sizeof(asn_DEF_GANSSTOD_GSMTimeAssociation_tags_1)
		/sizeof(asn_DEF_GANSSTOD_GSMTimeAssociation_tags_1[0]), /* 1 */
	0,	/* No PER visible constraints */
	asn_MBR_GANSSTOD_GSMTimeAssociation_1,
	6,	/* Elements count */
	&asn_SPC_GANSSTOD_GSMTimeAssociation_specs_1	/* Additional specs */
};

