/*
 * Generated by asn1c-0.9.22 (http://lionet.info/asn1c)
 * From ASN.1 module "RRLP-Components"
 * 	found in "../rrlp-components.asn"
 */

#ifndef	_AssistanceData_H_
#define	_AssistanceData_H_


#include <asn_application.h>

/* Including external dependencies */
#include "MoreAssDataToBeSent.h"
#include "ExtensionContainer.h"
#include <constr_SEQUENCE.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Forward declarations */
struct ReferenceAssistData;
struct MsrAssistData;
struct SystemInfoAssistData;
struct GPS_AssistData;
struct Rel98_AssistanceData_Extension;
struct Rel5_AssistanceData_Extension;

/* AssistanceData */
typedef struct AssistanceData {
	struct ReferenceAssistData	*referenceAssistData	/* OPTIONAL */;
	struct MsrAssistData	*msrAssistData	/* OPTIONAL */;
	struct SystemInfoAssistData	*systemInfoAssistData	/* OPTIONAL */;
	struct GPS_AssistData	*gps_AssistData	/* OPTIONAL */;
	MoreAssDataToBeSent_t	*moreAssDataToBeSent	/* OPTIONAL */;
	ExtensionContainer_t	*extensionContainer	/* OPTIONAL */;
	/*
	 * This type is extensible,
	 * possible extensions are below.
	 */
	struct Rel98_AssistanceData_Extension	*rel98_AssistanceData_Extension	/* OPTIONAL */;
	struct Rel5_AssistanceData_Extension	*rel5_AssistanceData_Extension	/* OPTIONAL */;
	
	/* Context for parsing across buffer boundaries */
	asn_struct_ctx_t _asn_ctx;
} AssistanceData_t;

/* Implementation */
extern asn_TYPE_descriptor_t asn_DEF_AssistanceData;

#ifdef __cplusplus
}
#endif

/* Referred external types */
#include "ReferenceAssistData.h"
#include "MsrAssistData.h"
#include "SystemInfoAssistData.h"
#include "GPS-AssistData.h"
#include "Rel98-AssistanceData-Extension.h"
#include "Rel5-AssistanceData-Extension.h"

#endif	/* _AssistanceData_H_ */
#include <asn_internal.h>
