/*
 * Generated by asn1c-0.9.22 (http://lionet.info/asn1c)
 * From ASN.1 module "ULP"
 * 	found in "../supl-ulp.asn"
 */

#include "UlpMessage.h"

static asn_per_constraints_t asn_PER_type_UlpMessage_constr_1 = {
	{ APC_CONSTRAINED | APC_EXTENSIBLE,  3,  3,  0,  7 }	/* (0..7,...) */,
	{ APC_UNCONSTRAINED,	-1, -1,  0,  0 },
	0, 0	/* No PER value map */
};
static asn_TYPE_member_t asn_MBR_UlpMessage_1[] = {
	{ ATF_NOFLAGS, 0, offsetof(struct UlpMessage, choice.msSUPLINIT),
		(ASN_TAG_CLASS_CONTEXT | (0 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_SUPLINIT,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"msSUPLINIT"
		},
	{ ATF_NOFLAGS, 0, offsetof(struct UlpMessage, choice.msSUPLSTART),
		(ASN_TAG_CLASS_CONTEXT | (1 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_SUPLSTART,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"msSUPLSTART"
		},
	{ ATF_NOFLAGS, 0, offsetof(struct UlpMessage, choice.msSUPLRESPONSE),
		(ASN_TAG_CLASS_CONTEXT | (2 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_SUPLRESPONSE,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"msSUPLRESPONSE"
		},
	{ ATF_NOFLAGS, 0, offsetof(struct UlpMessage, choice.msSUPLPOSINIT),
		(ASN_TAG_CLASS_CONTEXT | (3 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_SUPLPOSINIT,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"msSUPLPOSINIT"
		},
	{ ATF_NOFLAGS, 0, offsetof(struct UlpMessage, choice.msSUPLPOS),
		(ASN_TAG_CLASS_CONTEXT | (4 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_SUPLPOS,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"msSUPLPOS"
		},
	{ ATF_NOFLAGS, 0, offsetof(struct UlpMessage, choice.msSUPLEND),
		(ASN_TAG_CLASS_CONTEXT | (5 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_SUPLEND,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"msSUPLEND"
		},
	{ ATF_NOFLAGS, 0, offsetof(struct UlpMessage, choice.msDUMMY2),
		(ASN_TAG_CLASS_CONTEXT | (6 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_DUMMY,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"msDUMMY2"
		},
	{ ATF_NOFLAGS, 0, offsetof(struct UlpMessage, choice.msDUMMY3),
		(ASN_TAG_CLASS_CONTEXT | (7 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_DUMMY,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"msDUMMY3"
		},
};
static asn_TYPE_tag2member_t asn_MAP_UlpMessage_tag2el_1[] = {
    { (ASN_TAG_CLASS_CONTEXT | (0 << 2)), 0, 0, 0 }, /* msSUPLINIT at 37 */
    { (ASN_TAG_CLASS_CONTEXT | (1 << 2)), 1, 0, 0 }, /* msSUPLSTART at 38 */
    { (ASN_TAG_CLASS_CONTEXT | (2 << 2)), 2, 0, 0 }, /* msSUPLRESPONSE at 39 */
    { (ASN_TAG_CLASS_CONTEXT | (3 << 2)), 3, 0, 0 }, /* msSUPLPOSINIT at 40 */
    { (ASN_TAG_CLASS_CONTEXT | (4 << 2)), 4, 0, 0 }, /* msSUPLPOS at 41 */
    { (ASN_TAG_CLASS_CONTEXT | (5 << 2)), 5, 0, 0 }, /* msSUPLEND at 42 */
    { (ASN_TAG_CLASS_CONTEXT | (6 << 2)), 6, 0, 0 }, /* msDUMMY2 at 44 */
    { (ASN_TAG_CLASS_CONTEXT | (7 << 2)), 7, 0, 0 } /* msDUMMY3 at 46 */
};
static asn_CHOICE_specifics_t asn_SPC_UlpMessage_specs_1 = {
	sizeof(struct UlpMessage),
	offsetof(struct UlpMessage, _asn_ctx),
	offsetof(struct UlpMessage, present),
	sizeof(((struct UlpMessage *)0)->present),
	asn_MAP_UlpMessage_tag2el_1,
	8,	/* Count of tags in the map */
	0,
	8	/* Extensions start */
};
asn_TYPE_descriptor_t asn_DEF_UlpMessage = {
	"UlpMessage",
	"UlpMessage",
	CHOICE_free,
	CHOICE_print,
	CHOICE_constraint,
	CHOICE_decode_ber,
	CHOICE_encode_der,
	CHOICE_decode_xer,
	CHOICE_encode_xer,
	CHOICE_decode_uper,
	CHOICE_encode_uper,
	CHOICE_outmost_tag,
	0,	/* No effective tags (pointer) */
	0,	/* No effective tags (count) */
	0,	/* No tags (pointer) */
	0,	/* No tags (count) */
	&asn_PER_type_UlpMessage_constr_1,
	asn_MBR_UlpMessage_1,
	8,	/* Elements count */
	&asn_SPC_UlpMessage_specs_1	/* Additional specs */
};

