/*
 * Generated by asn1c-0.9.22 (http://lionet.info/asn1c)
 * From ASN.1 module "SUPL-START"
 * 	found in "../supl-start.asn"
 */

#ifndef	_SETCapabilities_H_
#define	_SETCapabilities_H_


#include <asn_application.h>

/* Including external dependencies */
#include "PosTechnology.h"
#include "PrefMethod.h"
#include "PosProtocol.h"
#include <constr_SEQUENCE.h>

#ifdef __cplusplus
extern "C" {
#endif

/* SETCapabilities */
typedef struct SETCapabilities {
	PosTechnology_t	 posTechnology;
	PrefMethod_t	 prefMethod;
	PosProtocol_t	 posProtocol;
	/*
	 * This type is extensible,
	 * possible extensions are below.
	 */
	
	/* Context for parsing across buffer boundaries */
	asn_struct_ctx_t _asn_ctx;
} SETCapabilities_t;

/* Implementation */
extern asn_TYPE_descriptor_t asn_DEF_SETCapabilities;

#ifdef __cplusplus
}
#endif

#endif	/* _SETCapabilities_H_ */
#include <asn_internal.h>
