/*
 * Generated by asn1c-0.9.22 (http://lionet.info/asn1c)
 * From ASN.1 module "RRLP-Components"
 * 	found in "../rrlp-components.asn"
 */

#ifndef	_SystemInfoAssistBTS_H_
#define	_SystemInfoAssistBTS_H_


#include <asn_application.h>

/* Including external dependencies */
#include <NULL.h>
#include "AssistBTSData.h"
#include <constr_CHOICE.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Dependencies */
typedef enum SystemInfoAssistBTS_PR {
	SystemInfoAssistBTS_PR_NOTHING,	/* No components present */
	SystemInfoAssistBTS_PR_notPresent,
	SystemInfoAssistBTS_PR_present
} SystemInfoAssistBTS_PR;

/* SystemInfoAssistBTS */
typedef struct SystemInfoAssistBTS {
	SystemInfoAssistBTS_PR present;
	union SystemInfoAssistBTS_u {
		NULL_t	 notPresent;
		AssistBTSData_t	 present;
	} choice;
	
	/* Context for parsing across buffer boundaries */
	asn_struct_ctx_t _asn_ctx;
} SystemInfoAssistBTS_t;

/* Implementation */
extern asn_TYPE_descriptor_t asn_DEF_SystemInfoAssistBTS;

#ifdef __cplusplus
}
#endif

#endif	/* _SystemInfoAssistBTS_H_ */
#include <asn_internal.h>
