/*
 * Generated by asn1c-0.9.22 (http://lionet.info/asn1c)
 * From ASN.1 module "ULP-Components"
 * 	found in "../supl-common.asn"
 */

#include "FrequencyInfo.h"

static asn_per_constraints_t asn_PER_type_fmodeSpecificInfo_constr_2 = {
	{ APC_CONSTRAINED | APC_EXTENSIBLE,  1,  1,  0,  1 }	/* (0..1,...) */,
	{ APC_UNCONSTRAINED,	-1, -1,  0,  0 },
	0, 0	/* No PER value map */
};
static asn_TYPE_member_t asn_MBR_fmodeSpecificInfo_2[] = {
	{ ATF_NOFLAGS, 0, offsetof(struct fmodeSpecificInfo, choice.fdd),
		(ASN_TAG_CLASS_CONTEXT | (0 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_FrequencyInfoFDD,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"fdd"
		},
	{ ATF_NOFLAGS, 0, offsetof(struct fmodeSpecificInfo, choice.tdd),
		(ASN_TAG_CLASS_CONTEXT | (1 << 2)),
		-1,	/* IMPLICIT tag at current level */
		&asn_DEF_FrequencyInfoTDD,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"tdd"
		},
};
static asn_TYPE_tag2member_t asn_MAP_fmodeSpecificInfo_tag2el_2[] = {
    { (ASN_TAG_CLASS_CONTEXT | (0 << 2)), 0, 0, 0 }, /* fdd at 122 */
    { (ASN_TAG_CLASS_CONTEXT | (1 << 2)), 1, 0, 0 } /* tdd at 123 */
};
static asn_CHOICE_specifics_t asn_SPC_fmodeSpecificInfo_specs_2 = {
	sizeof(struct fmodeSpecificInfo),
	offsetof(struct fmodeSpecificInfo, _asn_ctx),
	offsetof(struct fmodeSpecificInfo, present),
	sizeof(((struct fmodeSpecificInfo *)0)->present),
	asn_MAP_fmodeSpecificInfo_tag2el_2,
	2,	/* Count of tags in the map */
	0,
	2	/* Extensions start */
};
static /* Use -fall-defs-global to expose */
asn_TYPE_descriptor_t asn_DEF_fmodeSpecificInfo_2 = {
	"fmodeSpecificInfo",
	"fmodeSpecificInfo",
	CHOICE_free,
	CHOICE_print,
	CHOICE_constraint,
	CHOICE_decode_ber,
	CHOICE_encode_der,
	CHOICE_decode_xer,
	CHOICE_encode_xer,
	CHOICE_decode_uper,
	CHOICE_encode_uper,
	CHOICE_outmost_tag,
	0,	/* No effective tags (pointer) */
	0,	/* No effective tags (count) */
	0,	/* No tags (pointer) */
	0,	/* No tags (count) */
	&asn_PER_type_fmodeSpecificInfo_constr_2,
	asn_MBR_fmodeSpecificInfo_2,
	2,	/* Elements count */
	&asn_SPC_fmodeSpecificInfo_specs_2	/* Additional specs */
};

static asn_TYPE_member_t asn_MBR_FrequencyInfo_1[] = {
	{ ATF_NOFLAGS, 0, offsetof(struct FrequencyInfo, fmodeSpecificInfo),
		(ASN_TAG_CLASS_CONTEXT | (0 << 2)),
		+1,	/* EXPLICIT tag at current level */
		&asn_DEF_fmodeSpecificInfo_2,
		0,	/* Defer constraints checking to the member type */
		0,	/* No PER visible constraints */
		0,
		"fmodeSpecificInfo"
		},
};
static ber_tlv_tag_t asn_DEF_FrequencyInfo_tags_1[] = {
	(ASN_TAG_CLASS_UNIVERSAL | (16 << 2))
};
static asn_TYPE_tag2member_t asn_MAP_FrequencyInfo_tag2el_1[] = {
    { (ASN_TAG_CLASS_CONTEXT | (0 << 2)), 0, 0, 0 } /* fmodeSpecificInfo at 122 */
};
static asn_SEQUENCE_specifics_t asn_SPC_FrequencyInfo_specs_1 = {
	sizeof(struct FrequencyInfo),
	offsetof(struct FrequencyInfo, _asn_ctx),
	asn_MAP_FrequencyInfo_tag2el_1,
	1,	/* Count of tags in the map */
	0, 0, 0,	/* Optional elements (not needed) */
	0,	/* Start extensions */
	2	/* Stop extensions */
};
asn_TYPE_descriptor_t asn_DEF_FrequencyInfo = {
	"FrequencyInfo",
	"FrequencyInfo",
	SEQUENCE_free,
	SEQUENCE_print,
	SEQUENCE_constraint,
	SEQUENCE_decode_ber,
	SEQUENCE_encode_der,
	SEQUENCE_decode_xer,
	SEQUENCE_encode_xer,
	SEQUENCE_decode_uper,
	SEQUENCE_encode_uper,
	0,	/* Use generic outmost tag fetcher */
	asn_DEF_FrequencyInfo_tags_1,
	sizeof(asn_DEF_FrequencyInfo_tags_1)
		/sizeof(asn_DEF_FrequencyInfo_tags_1[0]), /* 1 */
	asn_DEF_FrequencyInfo_tags_1,	/* Same as above */
	sizeof(asn_DEF_FrequencyInfo_tags_1)
		/sizeof(asn_DEF_FrequencyInfo_tags_1[0]), /* 1 */
	0,	/* No PER visible constraints */
	asn_MBR_FrequencyInfo_1,
	1,	/* Elements count */
	&asn_SPC_FrequencyInfo_specs_1	/* Additional specs */
};

