/*
 * Generated by asn1c-0.9.22 (http://lionet.info/asn1c)
 * From ASN.1 module "RRLP-Components"
 * 	found in "../rrlp-components.asn"
 */

#ifndef	_Rel_7_MsrPosition_Rsp_Extension_H_
#define	_Rel_7_MsrPosition_Rsp_Extension_H_


#include <asn_application.h>

/* Including external dependencies */
#include "VelocityEstimate.h"
#include <constr_SEQUENCE.h>

#ifdef __cplusplus
extern "C" {
#endif

/* Forward declarations */
struct GANSSLocationInfo;
struct GANSSMeasureInfo;

/* Rel-7-MsrPosition-Rsp-Extension */
typedef struct Rel_7_MsrPosition_Rsp_Extension {
	VelocityEstimate_t	*velEstimate	/* OPTIONAL */;
	struct GANSSLocationInfo	*ganssLocationInfo	/* OPTIONAL */;
	struct GANSSMeasureInfo	*ganssMeasureInfo	/* OPTIONAL */;
	/*
	 * This type is extensible,
	 * possible extensions are below.
	 */
	
	/* Context for parsing across buffer boundaries */
	asn_struct_ctx_t _asn_ctx;
} Rel_7_MsrPosition_Rsp_Extension_t;

/* Implementation */
extern asn_TYPE_descriptor_t asn_DEF_Rel_7_MsrPosition_Rsp_Extension;

#ifdef __cplusplus
}
#endif

/* Referred external types */
#include "GANSSLocationInfo.h"
#include "GANSSMeasureInfo.h"

#endif	/* _Rel_7_MsrPosition_Rsp_Extension_H_ */
#include <asn_internal.h>
